//------------------------------------------------------------------------------
//	$Id: plusmap.js,v 1.10 2023/01/17 19:30:26 wolf Exp $
//------------------------------------------------------------------------------

/*jsl:option explicit*/
/*jsl:import OpenLayers.js*/
/*jsl:import geodetic.js*/
/*jsl:import grid_gk.js*/
/*jsl:import grid_utm.js*/
/*jsl:import grid_wgs.js*/
/*jsl:import layerchanger.js*/
/*jsl:import loadstatus.js*/
/*jsl:import measure_tool.js*/
/*jsl:import mouseposition_dm.js*/
/*jsl:import mouseposition_dms.js*/
/*jsl:import mouseposition_elevation.js*/
/*jsl:import mouseposition_gk.js*/
/*jsl:import mouseposition_utm.js*/
/*jsl:import nominatim_translation_de.js*/
/*jsl:import osrm5_analyzer.js*/
/*jsl:import popupmarker.js*/
/*jsl:import searchbox.js*/
/*jsl:import shading.js*/
/*jsl:import xpermalink.js*/

'use strict';

//------------------------------------------------------------------------------
// Die anonyme Onload-Function baut das Kartenobjekt aus Komponenten zusammen.
//------------------------------------------------------------------------------

window.onload = function () {

	//----------------------------------------------------------------------
	//	fix default passive==true in newer FF and Chrome
	//----------------------------------------------------------------------

	var oldWinAddEvtLis = window.addEventListener;
	window.addEventListener = function( name, listener, useCapture ){
		return oldWinAddEvtLis.apply( window, [ name, listener,
		{ capture: useCapture, passive: name != 'DOMMouseScroll' }]);
	};

	//----------------------------------------------------------------------
	//
	//	Utils
	//
	//----------------------------------------------------------------------

	var isLocal = document.location.hostname.indexOf('local') >=1;

	var parameters = OpenLayers.Util.getParameters();

	if (parameters.title) {

		window.document.title = parameters.title;
	}

	//----------------------------------------------------------------------
	//	Erzeuge Array mit Server-Resolutions
	//----------------------------------------------------------------------

	var genResolutions = function (maxZoom) {

		var resolution0 = 156543.03390625;
		var resolutions = [];

		for (var z=0; z<=maxZoom; z++) {

			resolutions.push(resolution0 / Math.pow(2, z));
		}

		return resolutions;
	};

	//----------------------------------------------------------------------
	//
	//	Karte
	//
	//----------------------------------------------------------------------

	var center = null;

	var map = new OpenLayers.Map ('map', {

		controls:[
			new OpenLayers.Control.Attribution({
				displayInLayerSwitcher: true,
				active: true
			}),
			new OpenLayers.Control.Navigation(),
			new OpenLayers.Control.PanZoomBar(),
			new OpenLayers.Control.Permalink()
		],
		displayProjection: new OpenLayers.Projection('EPSG:4326')
	});

	//----------------------------------------------------------------------
	//
	//	Steuerelemente
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	Tile-Lade-Status
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.LoadStatus({
		html: '<img src="i/ajax-loader-54x55.gif"/><br/>${layers}'
	}));

	//----------------------------------------------------------------------
	//	Entfernungs-Skala
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.ScaleLine({

		autoEnable: true,
		displayInLayerSwitcher: true,
		layerGroup: 'meas',
		permaId: 'Cs',
		title: 'Skalenanzeige'
	}));

	//----------------------------------------------------------------------
	//	Hoehenanzeige
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.MousePositionElevation({

		autoActivate: isLocal,
		displayInLayerSwitcher: true,
		layerGroup: 'elev',
		positionInLayerChanger: 1,
		permaId: 'Ce',
		title: 'H\xf6henanzeige',
		url: 'php/ele.php'
	}));

	//----------------------------------------------------------------------
	//	Kompass-Control
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control ({

		title: 'Kompass',
		layerGroup: 'meas',
		displayInLayerSwitcher: true,
		permaId: 'Cc',
		imgUrl: 'i/kompass-300x233.png',

		draw: function() {

			var img = document.createElement('img');

			var div = OpenLayers.Control.prototype.draw.apply(this);
			div.style.display='none';	// wenn beim Start ausgeblendet sein soll
			div.style.left='50%';
			div.style.marginLeft='-175px';
			div.style.marginTop='-175px';
			div.style.pointerEvents='none';
			div.style.top='50%';
			div.appendChild(img);

			return div;
		},

		activate: function() {

			this.div.style.display='';
			if (!OpenLayers.Control.prototype.activate.apply(this)) return false;
			this.div.firstChild.src = this.imgUrl;
			return true;
		},

		deactivate: function() {

			this.div.style.display='none';
			if (!OpenLayers.Control.prototype.deactivate.apply(this)) return false;
			return true;
		}
	}));

	//----------------------------------------------------------------------
	//	Koordinaten
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.MousePositionUTM({
		name: 'UTM',
		autoActivate: true,
		displayInLayerChanger: true,
		permaId: 'Mu',
		layerGroup: 'coor', radioGroup: 'utmc'
	}));

	map.addControl (new OpenLayers.Control.MousePositionUTM({
		name: 'UTMR',
		utmr: true,
		autoActivate: false,
		displayInLayerChanger: true,
		permaId: 'Mr',
		layerGroup: 'coor', radioGroup: 'utmc'
	}));

	map.addControl (new OpenLayers.Control.MousePositionGK({
		name: 'Gau\xdf Kr\xfcger',
		autoActivate: false,
		displayInLayerChanger: true,
		permaId: 'Mg',
		layerGroup: 'coor', radioGroup: 'utmc'
	}));

	map.addControl (new OpenLayers.Control.MousePosition({
		name: 'Grad',
		autoActivate: true,
		displayInLayerChanger: true,
		permaId: 'Md',
		layerGroup: 'coor', radioGroup: 'geoc'
	}));

	map.addControl (new OpenLayers.Control.MousePositionDM({
		name: 'Grad und Minuten',
		autoActivate: false,
		displayInLayerChanger: true,
		permaId: 'Mm',
		layerGroup: 'coor', radioGroup: 'geoc'
	}));

	map.addControl (new OpenLayers.Control.MousePositionDMS({
		name: 'Grad, Minute, Sekunden',
		autoActivate: false,
		displayInLayerChanger: true,
		permaId: 'Ms',
		layerGroup: 'coor', radioGroup: 'geoc'
	}));

	//----------------------------------------------------------------------
	//	Messwerkzeug
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.MeasureTool({

		title:			'Messwerkzeug [F9]',
		layerGroup:		'meas',

		cursor:			'url(i/crosshair-33x33.png) 16 16, crosshair',

		textLength:		'Orthodrome und loxodrome Entfernungen',
		textNoWay:		'Kein Weg gew\xe4hlt',
		textAnalyzing:		'Rechne ...',

		infoDivId:		'helptext-geodet',

		autoActivate:		false,
		radioGroup:		'analyzer',
		activationKeyCode:	120,		// F9

		resetOnDeactivate:	false,

		zoomToExtentKeyCode:	9,		// TAB
		clearKeyCode:		27,		// "Esc"
		//			33,		// "Page up"
		//			34,		// "Page down"
		toggleLineKeyCode:	35,		// "Ende"
		reverseKeyCode:		36,		// "Pos1"
		//			37,		// "Arrow Left"
		//			38,		// "Arrow Up"
		//			39,		// "Arrow Right"
		//			40,		// "Arrow Down"
		deleteLastKeyCode:	[8, 46],	// BACKSPACE, "Entf"
		restoreLastKeyCode:	45,		// "Einf"

		analyzer:		GeodeticAnalyzer,
		enableAnalyzer:		true
	}));

	//----------------------------------------------------------------------
	//	Hoehenmeterbestimmung fuer Messwerkzeug
	//----------------------------------------------------------------------

	var hoehenmesser = {

		attribution:
			'H\xf6hendaten von ' +
			'<a href="//www2.jpl.nasa.gov/srtm/">NASA SRTM</a>' +
			' via ' +
			'<a href="//www.opendem.info/">OpenDEM.info</a>',

		projection: new OpenLayers.Projection('EPSG:4326'),

		baseURL: '/c/geodata/ele.cgi?',

		formatDistance: function (measure, units) {

			if (units=='m') return measure.toFixed(1) + ' ' + units;
			if (units) return measure.toFixed(2) + ' ' + units;

			if (measure >= 1000) return (measure/1000).toFixed(2) + 'km';
			return (measure*1).toFixed(0) + 'm';
		},

		formatResult: function(text) {

			var data;

			try {
				data = JSON.parse(text);
			} catch(e) {
				return 'Interner Fehler (JSON)';
			}

			var result = [];

			if (data.error) {

				return 'H\xf6hendaten f\xfcr diesen Bereich nicht verf\xfcgbar.';
			}

			result.push('Entfernung: ' + this.formatDistance(data.distance) +
				' (' + this.formatDistance(data.flatDistance) + ')');

			if (data.ascend||data.descend) result.push('Auf/abstieg: ' + data.ascend + '/' + data.descend + 'm');
			if (data.maxEle) result.push('Min/max: ' + data.minEle + '/' + data.maxEle + 'm');

			return result.join('\n');
		},

		analyze: function (lonlats, onSuccess, onFailure) {

			var query = [];

			for (var i in lonlats) {

				var lonlat = lonlats[i];
				query.push (lonlat.lon.toFixed(5)+','+lonlat.lat.toFixed(5));
			}

			var url = this.baseURL + query.join('&');

			OpenLayers.Request.GET({

				async:	true,
				url:	url,

				success: function(request) {

					onSuccess (this.formatResult (request.responseText));
				},

				failure: function (request) {

					onFailure ('Status: ' + request.status + '\n' + request.responseText);
				},

				scope: this
			});
		}
	};

	//----------------------------------------------------------------------
	//	Messwerkzeug mit Hoehenmeterbestimmung
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.MeasureTool({

		title:			'H\xf6henmeter-Messwerkzeug',
		layerGroup:		'elev',
		positionInLayerChanger:	1000,

		cursor:			'url(i/crosshair-33x33.png) 16 16, crosshair',

		textLength:		'H\xf6henmeter-Messwerkzeug',
		textNoWay:		'Kein Weg gew\xe4hlt',
		textAnalyzing:		'Bestimme H\xf6henmeter ...',

		infoDivId:		'helptext-ele',

		autoActivate:		false,
		radioGroup:		'analyzer',

		resetOnDeactivate:	false,

		zoomToExtentKeyCode:	9,		// TAB
		clearKeyCode:		27,		// "Esc"
		//			33,		// "Page up"
		//			34,		// "Page down"
		analyzerKeyCode:	35,		// "Ende"
		reverseKeyCode:		36,		// "Pos1"
		//			37,		// "Arrow Left"
		//			38,		// "Arrow Up"
		//			39,		// "Arrow Right"
		//			40,		// "Arrow Down"
		deleteLastKeyCode:	[8, 46],	// BACKSPACE, "Entf"
		restoreLastKeyCode:	45,		// "Einf"

		analyzer:		hoehenmesser,
		enableAnalyzer:		true,

		displayClass:		'olControlMeasureTool-Ele'
	}));

	//----------------------------------------------------------------------
	//	Messwerkzeug mit OSRM-Interface
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.MeasureTool({

		title:			'Routing mit OSRM [F4]',
		layerGroup:		'rout',

		textLength:		'Routing mit der OSRM-API',
		textNoWay:		'Kein Weg gew\xe4hlt',
		textAnalyzing:		'Suche Route ...',

		infoDivId:		'helptext-osrm',

		autoActivate:		false,
		activationKeyCode:	115,		// F4
		radioGroup:		'analyzer',

		resetOnDeactivate:	false,

		zoomToExtentKeyCode:	9,		// TAB
		clearKeyCode:		27,		// "Esc"
		//			33,		// "Page up"
		//			34,		// "Page down"
		toggleLineKeyCode:	35,		// "Ende"
		reverseKeyCode:		36,		// "Pos1"
		//			37,		// "Arrow Left"
		//			38,		// "Arrow Up"
		//			39,		// "Arrow Right"
		//			40,		// "Arrow Down"
		restoreLastKeyCode:	45,		// "Einf"
		deleteLastKeyCode:	[8, 46],	// BACKSPACE, "Entf"

		enableAnalyzer:		true,

		markerStyleFirst: {
			externalGraphic: 'i/nadel-blau-28x28.png',
			graphicWidth:	28,
			graphicHeight:	28,
			graphicXOffset:	0,
			graphicYOffset:	-28,
			graphicZIndex:	5,
			title: 'Click to select, Strg-Click to delete',
			cursor: 'move'
		},

		markerStyle: {
			externalGraphic: 'i/nadel-gelb-24x25.png',
			graphicWidth:	24,
			graphicHeight:	25,
			graphicXOffset:	0,
			graphicYOffset:	-25,
			graphicZIndex:	4,
			title: 'Strg-Click to delete',
			cursor: 'move'
		},

		markerStyleLast: {
			externalGraphic: 'i/nadel-rot-26x30.png',
			graphicWidth:	26,
			graphicHeight:	30,
			graphicXOffset:	0,
			graphicYOffset:	-30,
			graphicZIndex:	6,
			title: 'Strg-Click to delete',
			cursor: 'move'
		},

		markerStyleAux: {
			graphicName: 'circle',
			pointRadius: 4,
			strokeWidth: 1,
			strokeColor: 'blue',
			fillColor: 'blue',
			fillOpacity: 0.3,
			graphicZIndex:	2,
			cursor: 'move'
		},

		lineStyle: {
			strokeColor: 'blue',
			strokeWidth: 5,
			strokeOpacity: 0.3,
			graphicZIndex:	1
		},

		analyzer:	OSRM_ANALYZER,

		displayClass:		'olControlMeasureTool-Osrm'
	}));

	//----------------------------------------------------------------------
	//	Layer changer
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.LayerChanger({

		layerGroups: [
			{id: 'main', name: 'Kartenhintergrund'},
			{id: 'elev', name: 'H\xf6hendarstellung', hidden: true},
			{id: 'grid', name: 'Kartengitter', hidden: true},
			{id: 'data', name: 'Thematische Erg\xe4nzungen', hidden: true},
			{id: 'meas', name: 'Messen', hidden: true},
			{id: 'rout', name: 'Routing', hidden: true},
			{id: 'ctrl', name: 'Steuerelemente', hidden: true},
			{id: 'coor', name: 'Koordinatenanzeige', hidden: true},
			{id: 'link', name: 'Verweise auf andere Karten', hidden: true},
			{id: 'edit', name: 'Karte bearbeiten', hidden: true}
		],

		//--------------------------------------------------------------
		//	Optional: auto close groups if overflows
		//--------------------------------------------------------------

		enableAutoClose: true,
		heightMargin: 90,

		//--------------------------------------------------------------
		//	Optional: maximale Zahl geoeffneter Gruppen
		//--------------------------------------------------------------

		maxGroupsOpen: null,

		//--------------------------------------------------------------
		//	Optionaler Paramter zur Anzeige von Controls:
		//--------------------------------------------------------------
		//	controls: true	- waehlt *alle* sichtbaren Controls
		//	controls: []	- waehlt die Controls aus der Liste
		//
		//	In diesem Array kann (auch gemischt) angeben werden:
		//	* das Control (das, was an addControl uebergeben wird),
		//	* ein Klassenname als String, z.B.: 'OpenLayers.Control.Zoom'
		//--------------------------------------------------------------
		//	ACHTUNG: der LayerChanger zeigt nur die Controls,
		//	die im Code *vorher* definiert sind.
		//--------------------------------------------------------------

		controls: [],

		//--------------------------------------------------------------
		//	Optional: position of entries w/o positionInLayerChanger
		//--------------------------------------------------------------

		defaultPosition: 100,

		//--------------------------------------------------------------
		//	Optional: Buttons zur Steuerung der Transparenz
		//--------------------------------------------------------------

		withOpacityButtons: true,

		//--------------------------------------------------------------
		//	Optional: Buttons zum Zoom auf einen Layer
		//--------------------------------------------------------------

		withFullZoomButton: true,

		//--------------------------------------------------------------
		//	Optional: Keycode zum Schalten des Layerchangers
		//--------------------------------------------------------------

		toggleKeyCode: 119,	// 119 = F8

		//--------------------------------------------------------------
		//	Optional: Hoehe oder Abstand zum unteren Fensterrand
		//--------------------------------------------------------------

		//maxHeight: 200,
		minBottom: 100,

		//--------------------------------------------------------------
		//	Optional: Kartenlinks
		//--------------------------------------------------------------

		mapLinks: [{
			label: 'OpenStreetMap',
			title: 'Gleicher Bereich auf der OSM Hauptkarte',
			baseURL: 'https://www.openstreetmap.org/#map={zoom}/{lat}/{lon}',
			target: 'osm'
		}, {
			label: 'OpenStreetMap mit Notes',
			baseURL: 'https://www.openstreetmap.org/?layers=N#{zoom}/{lat}/{lon}',
			target: 'osm'
		}, {
			label: 'Topografische Karte von Maxbe',
			baseURL: 'https://geo.dianacht.de/topo/?zoom={zoom}&lat={lat}&lon={lon}',
			bounds: new OpenLayers.Bounds(10.00, 47.00, 13.25, 48.65),
			target: 'topo'
		}, {
			label: 'Historic.place',
			baseURL: 'https://gk.historic.place/historische_objekte/?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'hist'
		}, {
			label: 'Wanderwege',
			baseURL: 'https://hiking.waymarkedtrails.org/de/?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'lhm'
		}, {
			label: 'Radwege',
			baseURL: 'https://cycling.waymarkedtrails.org/de/?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'lhm'
		}, {
			label: 'Mountainbike-Routen',
			baseURL: 'https://mtb.waymarkedtrails.org/de/?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'lhm'
/*
		}, {
			label: 'Hikebikemap.de',
			baseURL: 'http://hikebikemap.de?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'hbm'
*/
		}, {
			label: 'Verkehrskarte',
			title: 'OSM Hauptkarte mit Verkehrskarten-Layer',
			baseURL: 'https://www.openstreetmap.org/#map={zoom}/{lat}/{lon}&layers=T',
			target: 'osm'
		}, {
			label: '\xd6PNV-Karte',
			baseURL: 'https://www.\xf6pnvkarte.de/?zoom={zoom}&lat={lat}&lon={lon}',
			target: 'oepnv'
		}, {
			layerGroup: 'edit',
			label: 'Bearbeiten JOSM',
			baseURL: 'https://127.0.0.1:8111/load_and_zoom?left={left}&bottom={bottom}&right={right}&top={top}',
			minZoom: 12,
			target: 'josm'
		}, {
			layerGroup: 'edit',
			label: 'Bearbeiten mit Id',
			baseURL: 'https://www.openstreetmap.org/edit?editor=id&zoom={zoom}&lat={lat}&lon={lon}',
			minZoom: 14,
			target: 'edit'
		}, {
			layerGroup: 'edit',
			label: 'Bearbeiten mit Potlatch',
			baseURL: 'https://www.openstreetmap.org/edit?editor=potlatch2&zoom={zoom}&lat={lat}&lon={lon}',
			minZoom: 14,
			target: 'edit'
		}]
	}));

	//----------------------------------------------------------------------
	//	SearchBox
	//----------------------------------------------------------------------

	map.addControl (new OpenLayers.Control.SearchBox({

		autoActivate: true,
		autoClose: false,
		displayInLayerChanger: true,
		resultMinZoom: 16,
		title: 'Suchfeld'
	}));

	//----------------------------------------------------------------------
	//
	//	Baselayer
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	Mapnik
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.OSM('Mapnik', [
		'https://a.tile.openstreetmap.org/${z}/${x}/${y}.png',
		'https://b.tile.openstreetmap.org/${z}/${x}/${y}.png',
		'https://c.tile.openstreetmap.org/${z}/${x}/${y}.png'
	], {
		numZoomLevels: 20,
		permaId: 'Bm',
		positionInLayerChanger:	2,
		serverResolutions: genResolutions(20)
	}));

	//----------------------------------------------------------------------
	//	openstreetmap.de
	//----------------------------------------------------------------------

	map.addLayer( new OpenLayers.Layer.OSM( 'Mapnik deutsch', [
		'https://a.tile.openstreetmap.de/${z}/${x}/${y}.png',
		'https://b.tile.openstreetmap.de/${z}/${x}/${y}.png',
		'https://c.tile.openstreetmap.de/${z}/${x}/${y}.png'
	], {
		numZoomLevels: 20,
		permaId: 'Bg',
		positionInLayerChanger:	2,
		serverResolutions: genResolutions(20)
	}));

	//----------------------------------------------------------------------
	//	ESRI Luftbild
	//----------------------------------------------------------------------

	if (isLocal) map.addLayer(new OpenLayers.Layer.XYZ('Esri Satellite', [
'https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/${z}/${y}/${x}.jpg'
	], {
		attribution: 'Satellite image \xa9 ESRI',
		displayOutsideMaxExtent: true,
		isBaseLayer: true,
		numZoomLevels: 22,
		permaId: 'Bsr',
		positionInLayerChanger:	2,
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transitionEffect: 'resize',
		transparent: false,
		type: 'png',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//	Opentopo
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.XYZ('OpenTopoMap', [
		'https://a.tile.opentopomap.org/${z}/${x}/${y}.png',
		'https://b.tile.opentopomap.org/${z}/${x}/${y}.png',
		'https://c.tile.opentopomap.org/${z}/${x}/${y}.png'
	], {
		attribution:
'Kartendarstellung: \xa9 <a target"_blank" href="//opentopomap.org/">OpenTopoMap</a>',
		displayOutsideMaxExtent: true,
		isBaseLayer: true,
		numZoomLevels: 18,
		permaId: 'Bot',
		positionInLayerChanger:	22,
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transparent: false,
		visibility: false
	}));

	//----------------------------------------------------------------------
	//	OePNV-Karte
	//----------------------------------------------------------------------

	map.addLayer( new OpenLayers.Layer.XYZ( '\xd6PNV-Karte', [
		'https://tileserver.memomaps.de/tilegen/${z}/${x}/${y}.png'
	], {
		attribution:
'Tiles from <a target="_blank" href="https://www.xn--pnvkarte-m4a.de">&#xd6;PNV-Karte</a>',
		displayOutsideMaxExtent: true,
		isBaseLayer: true,
		permaId: 'Boe',
		positionInLayerChanger:	43,
		numZoomLevels: 20,
		serverResolutions: genResolutions( 17 ),
		sphericalMercator: true,
		transparent: false,
		visibility: false
	}));

/*
	//----------------------------------------------------------------------
	//	Hikebikemap
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.XYZ('Hikebikemap.de', [
		'https://a.tiles.wmflabs.org/hikebike/${z}/${x}/${y}.png',
		'https://b.tiles.wmflabs.org/hikebike/${z}/${x}/${y}.png',
		'https://c.tiles.wmflabs.org/hikebike/${z}/${x}/${y}.png'
	], {
		attribution:
'Tiles from <a target="_blank" href="//hikebikemap.org/">hikebikemap.org</a>',
		displayOutsideMaxExtent: true,
		isBaseLayer: true,
		permaId: 'Bhb',
		positionInLayerChanger:	41,
		numZoomLevels: 20,
		serverResolutions: genResolutions(20),
		sphericalMercator: true,
		transparent: false,
		visibility: false
	}));
*/

	//----------------------------------------------------------------------
	//	Topographische Karte von Maxbe
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.XYZ('Topomap von Maxbe', [
		'https://tiles2.dianacht.de/mapproxy/tiles/1.0.0/Topo/mercator/${z}/${x}/${y}.png'
	], {
		attribution:
'Kartendarstellung: \xa9 <a target"_blank" href="//geo.dianacht.de/topo/">Max Berger</a>',
		displayOutsideMaxExtent: true,
		isBaseLayer: true,
		numZoomLevels: 18,
		permaId: 'Btm',
		positionInLayerChanger:	21,
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transparent: false,
		type: 'png',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//
	//	Overlays
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	Shading
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.Shading('Schummerung', null, {
		layerGroup: 'elev',
		permaId: 'Sx',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//	Hoehenlinien
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.XYZ( 'H\xf6henlinien (Z12-14)', [
		'https://tiles.historic.place/ele/de/${z}/${x}/${y}.png'
	], {
		alpha: true,
		isBaseLayer: false,
		layerGroup: 'elev',
		opacity: 0.5,
		permaId: 'Och',
		maxResolution: 40, // ab Zoom 12
		serverResolutions: genResolutions(14),
		sphericalMercator: true,
		visibility: false
	}));

	//----------------------------------------------------------------------
	//
	//	Raster
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	UTM-Grid
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.GridUTM ('UTM-Gitter (ab Z9)', {
		layerGroup: 'grid',
		radioGroup: 'grid',
		permaId: 'Gu',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//	GK-Grid
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.GridGK ('Gau\xdf-Kr\xfcger-Gitter (ab Z9)', {
		layerGroup: 'grid',
		radioGroup: 'grid',
		permaId: 'Gg',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//	WGS-Grid
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.GridWGS('Koordinatenraster', {
		layerGroup: 'grid',
		radioGroup: 'grid',
		permaId: 'Gg',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//
	//	Overlays - Flaechen
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	Verwaltungsgrenzen (nur RoBerge)
	//----------------------------------------------------------------------

	if (false) map.addLayer (new OpenLayers.Layer.Vector ('Verwaltungs-Grenzen', {

		layerGroup: 'ovly',
		permaId: 'Oaa',
		visibility: false,

		//--------------------------------------------------------------
		//	default style
		//--------------------------------------------------------------

		preFeatureInsert: function(feature) {

			switch ((feature.attributes||{}).admin_level) {

			case '6':
				feature.style = {
					strokeWidth: 11,
					strokeColor: 'blue',
					strokeOpacity: 1.0
				};
				break;

			case '7':
				feature.style = {
					strokeWidth: 7,
					strokeColor: 'yellow',
					strokeOpacity: 1.0,
					strokeDashstyle: 'dot'
				};
				break;

			default:
				feature.style = {
					strokeWidth: 3,
					strokeColor: 'red',
					strokeOpacity: 0.9
				};
				break;
			}
		},

		//--------------------------------------------------------------
		//	Strategy.BBOX: reload on pan or zoom
		//--------------------------------------------------------------

		strategies: [new OpenLayers.Strategy.BBOX({resFactor: 1.5, ratio: 1.2 })],

		//--------------------------------------------------------------
		//	Get new track per HTTP request
		//	value of mindist parameter is changed by moveTo()
		//--------------------------------------------------------------

		protocol: new OpenLayers.Protocol.HTTP({

			url: '/c/geodata/bbox.cgi',

			params: {
				db:	'ro_admin_6_8',
				limit:	2500,
				zoom:	0
			},

			format: new OpenLayers.Format.GeoJSON()
		}),

		//--------------------------------------------------------------
		//	Discard strategie cache on zoom change
		//--------------------------------------------------------------

		moveTo: function (bounds, zoomChanged, dragging) {

			if (zoomChanged) {

				var mapZoom = this.map.getZoom();
				var newZoom = mapZoom>=9 ? Math.min(mapZoom, 12) : 0;

				if (newZoom != this.protocol.params.zoom) {

					this.strategies[0].bounds = null;
				}

				this.protocol.params.zoom = newZoom;
			}

			OpenLayers.Layer.Vector.prototype.moveTo.apply(this, arguments);
		}
	}));

	//----------------------------------------------------------------------
	//	Postleitzahlen
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.Vector ('PLZ-Bereiche DACH [Z12]', {

		layerGroup: 'ovly',
		permaId: 'Oap',
		visibility: false,

		//--------------------------------------------------------------
		//	default style
		//--------------------------------------------------------------

		styleMap: new OpenLayers.StyleMap ({

			'default': new OpenLayers.Style ({

				strokeColor: 'black',
				strokeWidth: 1,
				strokeOpacity: 0.7
			})
		}),

		//--------------------------------------------------------------
		//	Strategy.BBOX: reload on pan or zoom
		//--------------------------------------------------------------

		strategies: [new OpenLayers.Strategy.BBOX({resFactor: 1.5 })],

		//--------------------------------------------------------------
		//	Get new track per HTTP request
		//	value of mindist parameter is changed by moveTo()
		//--------------------------------------------------------------

		protocol: new OpenLayers.Protocol.HTTP({

			url: '/c/geodata/bbox.cgi',

			params: {
				db:	'postal_dach',
				limit:	2500,
				zoom:	0
			},

			format: new OpenLayers.Format.GeoJSON()
		}),

		//--------------------------------------------------------------
		//	Discard strategie cache on zoom change
		//--------------------------------------------------------------

		moveTo: function (bounds, zoomChanged, dragging) {

			if (zoomChanged) {

				var mapZoom = this.map.getZoom();
				var newZoom = mapZoom>=12 ? Math.min(mapZoom, 14) : 0;

				if (newZoom != this.protocol.params.zoom) {

					this.strategies[0].bounds = null;
				}

				this.protocol.params.zoom = newZoom;
			}

			OpenLayers.Layer.Vector.prototype.moveTo.apply(this, arguments);
		}
	}));

	//----------------------------------------------------------------------
	//
	//	Overlays - Linien
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//	Waymarked trails
	//----------------------------------------------------------------------

	map.addLayer (new OpenLayers.Layer.XYZ('Wanderwege', [
		'https://tile.waymarkedtrails.org/hiking/${z}/${x}/${y}.png'
	], {
		attribution:
'Routen by <a target="_blank" href="//hiking.waymarkedtrails.org/de/">Waymarked Trails</a>',
		description: 'Lonvias Waymarked Trails: Wanderwege',
		isBaseLayer: false,
		layerGroup: 'ovly',
		numZoomLevels: 21,
		permaId: 'Orh',
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transparent: true,
		type: 'png',
		visibility: false
	}));

	map.addLayer (new OpenLayers.Layer.XYZ('Radwege', [
		'https://tile.waymarkedtrails.org/cycling/${z}/${x}/${y}.png'
	], {
		attribution:
'Routen by <a target="_blank" href="//hiking.waymarkedtrails.org/de/">Waymarked Trails</a>',
		description: 'Lonvias Waymarked Trails: Radwege',
		isBaseLayer: false,
		layerGroup: 'ovly',
		numZoomLevels: 21,
		permaId: 'Orc',
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transparent: true,
		type: 'png',
		visibility: false
	}));

	map.addLayer (new OpenLayers.Layer.XYZ('Mountainbike-Routen', [
		'https://tile.waymarkedtrails.org/mtb/${z}/${x}/${y}.png'
	], {
		attribution:
'Routen by <a target="_blank" href="//hiking.waymarkedtrails.org/de/">Waymarked Trails</a>',
		description: 'Lonvias Waymarked Trails: Mountainbike-Routen',
		isBaseLayer: false ,
		layerGroup: 'ovly',
		numZoomLevels: 21,
		permaId: 'Orm',
		serverResolutions: genResolutions(18),
		sphericalMercator: true,
		transparent: true,
		type: 'png',
		visibility: false
	}));

	//----------------------------------------------------------------------
	//
	//	Overlays - Punkte
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//
	//	Marker
	//
	//----------------------------------------------------------------------

	//----------------------------------------------------------------------
	//      Benannter Marker-Layer (wenn mlon und mlat angegeben)
	//----------------------------------------------------------------------

	var mlon = parseFloat(parameters.mlon);
	var mlat = parseFloat(parameters.mlat);

	if (!isNaN(mlon) && !isNaN(mlat)) {

		var markerLayer = new OpenLayers.Layer.Markers('Marker');
		map.addLayer (markerLayer);

		center = new OpenLayers.LonLat(mlon, mlat).
			transform(map.displayProjection, map.getProjectionObject());

		markerLayer.addMarker(new OpenLayers.Marker(center));
	}

	//----------------------------------------------------------------------
	//
	//	Stelle bestimmten Bereich an maximaler Groesse dar
	//
	//----------------------------------------------------------------------

	if (!map.getCenter()) {

		if (center) {

			map.setCenter (center, parameters.zoom || 13);

		} else {

			map.zoomToExtent(
				new OpenLayers.Bounds(5.88, 47.26, 15.04, 54.91).
					transform(new OpenLayers.Projection("EPSG:4326"),
					map.getProjectionObject()));
		}
	}
};

//------------------------------------------------------------------------------
//	$Id: plusmap.js,v 1.10 2023/01/17 19:30:26 wolf Exp $
//------------------------------------------------------------------------------
